@@ -123,6 +123,8 @@ array_size (Dwarf_Die *die, Dwarf_Word *size,
 		    case DW_LANG_Fortran77:
 		    case DW_LANG_Fortran90:
 		    case DW_LANG_Fortran95:
+		    case DW_LANG_Fortran03:
+		    case DW_LANG_Fortran08:
 		    case DW_LANG_Pascal83:
 		    case DW_LANG_Modula2:
 		    case DW_LANG_PL1:
@@ -228,9 +230,7 @@ aggregate_size (Dwarf_Die *die, Dwarf_Word *size, Dwarf_Die *type_mem)
 }
 
 int
-dwarf_aggregate_size (die, size)
-     Dwarf_Die *die;
-     Dwarf_Word *size;
+dwarf_aggregate_size (Dwarf_Die *die, Dwarf_Word *size)
 {
   Dwarf_Die type_mem;
 
