@@ -57,24 +57,6 @@ cu_free (void *arg)
 }
 
 
-#if USE_ZLIB
-void
-internal_function
-__libdw_free_zdata (Dwarf *dwarf)
-{
-  unsigned int gzip_mask = dwarf->sectiondata_gzip_mask;
-  while (gzip_mask != 0)
-    {
-      int i = ffs (gzip_mask);
-      assert (i > 0);
-      --i;
-      assert (i < IDX_last);
-      free (dwarf->sectiondata[i]);
-      gzip_mask &= ~(1U << i);
-    }
-}
-#endif
-
 int
 dwarf_end (Dwarf *dwarf)
 {
@@ -110,14 +92,16 @@ dwarf_end (Dwarf *dwarf)
       /* Free the pubnames helper structure.  */
       free (dwarf->pubnames_sets);
 
-      __libdw_free_zdata (dwarf);
-
       /* Free the ELF descriptor if necessary.  */
       if (dwarf->free_elf)
 	elf_end (dwarf->elf);
 
       /* Free the fake location list CU.  */
-      free (dwarf->fake_loc_cu);
+      if (dwarf->fake_loc_cu != NULL)
+	{
+	  cu_free (dwarf->fake_loc_cu);
+	  free (dwarf->fake_loc_cu);
+	}
 
       /* Free the context descriptor.  */
       free (dwarf);
