@@ -1,5 +1,5 @@
 /* Return line number information of CU.
-   Copyright (C) 2004-2010, 2013, 2014 Red Hat, Inc.
+   Copyright (C) 2004-2010, 2013, 2014, 2015 Red Hat, Inc.
    This file is part of elfutils.
    Written by Ulrich Drepper <drepper@redhat.com>, 2004.
 
@@ -78,6 +78,74 @@ compare_lines (const void *a, const void *b)
     : 0;
 }
 
+struct line_state
+{
+  Dwarf_Word addr;
+  unsigned int op_index;
+  unsigned int file;
+  int64_t line;
+  unsigned int column;
+  uint_fast8_t is_stmt;
+  bool basic_block;
+  bool prologue_end;
+  bool epilogue_begin;
+  unsigned int isa;
+  unsigned int discriminator;
+  struct linelist *linelist;
+  size_t nlinelist;
+  unsigned int end_sequence;
+};
+
+static inline void
+run_advance_pc (struct line_state *state, unsigned int op_advance,
+                uint_fast8_t minimum_instr_len, uint_fast8_t max_ops_per_instr)
+{
+  state->addr += minimum_instr_len * ((state->op_index + op_advance)
+				      / max_ops_per_instr);
+  state->op_index = (state->op_index + op_advance) % max_ops_per_instr;
+}
+
+static inline bool
+add_new_line (struct line_state *state, struct linelist *new_line)
+{
+  /* Set the line information.  For some fields we use bitfields,
+     so we would lose information if the encoded values are too large.
+     Check just for paranoia, and call the data "invalid" if it
+     violates our assumptions on reasonable limits for the values.  */
+  new_line->next = state->linelist;
+  new_line->sequence = state->nlinelist;
+  state->linelist = new_line;
+  ++(state->nlinelist);
+
+  /* Set the line information.  For some fields we use bitfields,
+     so we would lose information if the encoded values are too large.
+     Check just for paranoia, and call the data "invalid" if it
+     violates our assumptions on reasonable limits for the values.  */
+#define SET(field)						      \
+  do {								      \
+     new_line->line.field = state->field;			      \
+     if (unlikely (new_line->line.field != state->field))	      \
+       return true;						      \
+   } while (0)
+
+  SET (addr);
+  SET (op_index);
+  SET (file);
+  SET (line);
+  SET (column);
+  SET (is_stmt);
+  SET (basic_block);
+  SET (end_sequence);
+  SET (prologue_end);
+  SET (epilogue_begin);
+  SET (isa);
+  SET (discriminator);
+
+#undef SET
+
+  return false;
+}
+
 static int
 read_srclines (Dwarf *dbg,
 	       const unsigned char *linep, const unsigned char *lineendp,
@@ -86,14 +154,55 @@ read_srclines (Dwarf *dbg,
 {
   int res = -1;
 
-  struct linelist *linelist = NULL;
-  size_t nlinelist = 0;
+  size_t nfilelist = 0;
+  unsigned int ndirlist = 0;
 
-  /* If there are a large number of lines don't blow up the stack.
-     Keep track of the last malloced linelist record and free them
-     through the next pointer at the end.  */
+  struct filelist null_file =
+    {
+      .info =
+      {
+	.name = "???",
+	.mtime = 0,
+	.length = 0
+      },
+      .next = NULL
+    };
+  struct filelist *filelist = &null_file;
+
+  /* If there are a large number of lines, files or dirs don't blow up
+     the stack.  Stack allocate some entries, only dynamically malloc
+     when more than MAX.  */
 #define MAX_STACK_ALLOC 4096
-  struct linelist *malloc_linelist = NULL;
+#define MAX_STACK_LINES MAX_STACK_ALLOC
+#define MAX_STACK_FILES (MAX_STACK_ALLOC / 4)
+#define MAX_STACK_DIRS  (MAX_STACK_ALLOC / 16)
+
+  struct dirlist
+  {
+    const char *dir;
+    size_t len;
+  };
+  struct dirlist dirstack[MAX_STACK_DIRS];
+  struct dirlist *dirarray = dirstack;
+
+  /* We are about to process the statement program.  Initialize the
+     state machine registers (see 6.2.2 in the v2.1 specification).  */
+  struct line_state state =
+    {
+      .linelist = NULL,
+      .nlinelist = 0,
+      .addr = 0,
+      .op_index = 0,
+      .file = 1,
+      /* We only store int but want to check for overflow (see SET above).  */
+      .line = 1,
+      .column = 0,
+      .basic_block = false,
+      .prologue_end = false,
+      .epilogue_begin = false,
+      .isa = 0,
+      .discriminator = 0
+    };
 
   if (unlikely (linep + 4 > lineendp))
     {
@@ -170,65 +279,69 @@ read_srclines (Dwarf *dbg,
 
   /* First comes the list of directories.  Add the compilation
      directory first since the index zero is used for it.  */
-  struct dirlist
-  {
-    const char *dir;
-    size_t len;
-    struct dirlist *next;
-  } comp_dir_elem =
+  struct dirlist comp_dir_elem =
     {
       .dir = comp_dir,
       .len = comp_dir ? strlen (comp_dir) : 0,
-      .next = NULL
     };
-  struct dirlist *dirlist = &comp_dir_elem;
-  unsigned int ndirlist = 1;
+  ndirlist = 1;
 
-  // XXX Directly construct array to conserve memory?
-  while (*linep != 0)
+  /* First count the entries.  */
+  const unsigned char *dirp = linep;
+  while (*dirp != 0)
     {
-      struct dirlist *new_dir =
-	(struct dirlist *) alloca (sizeof (*new_dir));
-
-      new_dir->dir = (char *) linep;
-      uint8_t *endp = memchr (linep, '\0', lineendp - linep);
+      uint8_t *endp = memchr (dirp, '\0', lineendp - dirp);
       if (endp == NULL)
 	goto invalid_data;
-      new_dir->len = endp - linep;
-      new_dir->next = dirlist;
-      dirlist = new_dir;
       ++ndirlist;
+      dirp = endp + 1;
+    }
+
+  /* Arrange the list in array form.  */
+  if (ndirlist >= MAX_STACK_DIRS)
+    {
+      dirarray = (struct dirlist *) malloc (ndirlist * sizeof (*dirarray));
+      if (unlikely (dirarray == NULL))
+	{
+	no_mem:
+	  __libdw_seterrno (DWARF_E_NOMEM);
+	  goto out;
+	}
+    }
+  dirarray[0] = comp_dir_elem;
+  for (unsigned int n = 1; n < ndirlist; n++)
+    {
+      dirarray[n].dir = (char *) linep;
+      uint8_t *endp = memchr (linep, '\0', lineendp - linep);
+      assert (endp != NULL);
+      dirarray[n].len = endp - linep;
       linep = endp + 1;
     }
   /* Skip the final NUL byte.  */
   ++linep;
 
-  /* Rearrange the list in array form.  */
-  struct dirlist **dirarray
-    = (struct dirlist **) alloca (ndirlist * sizeof (*dirarray));
-  for (unsigned int n = ndirlist; n-- > 0; dirlist = dirlist->next)
-    dirarray[n] = dirlist;
+  /* Allocate memory for a new file.  For the first MAX_STACK_FILES
+     entries just return a slot in the preallocated stack array.  */
+  struct filelist flstack[MAX_STACK_FILES];
+#define NEW_FILE() ({							\
+  struct filelist *fl = (nfilelist < MAX_STACK_FILES			\
+			   ? &flstack[nfilelist]			\
+			   : malloc (sizeof (struct filelist)));	\
+  if (unlikely (fl == NULL))						\
+    goto no_mem;							\
+  ++nfilelist;								\
+  fl->next = filelist;							\
+  filelist = fl;							\
+  fl; })
 
   /* Now read the files.  */
-  struct filelist null_file =
-    {
-      .info =
-      {
-	.name = "???",
-	.mtime = 0,
-	.length = 0
-      },
-      .next = NULL
-    };
-  struct filelist *filelist = &null_file;
-  unsigned int nfilelist = 1;
+  nfilelist = 1;
 
   if (unlikely (linep >= lineendp))
     goto invalid_data;
   while (*linep != 0)
     {
-      struct filelist *new_file =
-	(struct filelist *) alloca (sizeof (*new_file));
+      struct filelist *new_file = NEW_FILE ();
 
       /* First comes the file name.  */
       char *fname = (char *) linep;
@@ -255,22 +368,22 @@ read_srclines (Dwarf *dbg,
       else
 	{
 	  new_file->info.name = libdw_alloc (dbg, char, 1,
-					     dirarray[diridx]->len + 1
+					     dirarray[diridx].len + 1
 					     + fnamelen + 1);
 	  char *cp = new_file->info.name;
 
-	  if (dirarray[diridx]->dir != NULL)
+	  if (dirarray[diridx].dir != NULL)
 	    {
 	      /* This value could be NULL in case the DW_AT_comp_dir
 		 was not present.  We cannot do much in this case.
 		 The easiest thing is to convert the path in an
 		 absolute path.  */
-	      cp = stpcpy (cp, dirarray[diridx]->dir);
+	      cp = stpcpy (cp, dirarray[diridx].dir);
 	    }
 	  *cp++ = '/';
 	  strcpy (cp, fname);
 	  assert (strlen (new_file->info.name)
-		  < dirarray[diridx]->len + 1 + fnamelen + 1);
+		  < dirarray[diridx].len + 1 + fnamelen + 1);
 	}
 
       /* Next comes the modification time.  */
@@ -282,10 +395,6 @@ read_srclines (Dwarf *dbg,
       if (unlikely (linep >= lineendp))
 	goto invalid_data;
       get_uleb128 (new_file->info.length, linep, lineendp);
-
-      new_file->next = filelist;
-      filelist = new_file;
-      ++nfilelist;
     }
   /* Skip the final NUL byte.  */
   ++linep;
@@ -297,80 +406,30 @@ read_srclines (Dwarf *dbg,
       goto out;
     }
 
-  /* We are about to process the statement program.  Initialize the
-     state machine registers (see 6.2.2 in the v2.1 specification).  */
-  Dwarf_Word addr = 0;
-  unsigned int op_index = 0;
-  unsigned int file = 1;
-  int line = 1;
-  unsigned int column = 0;
-  uint_fast8_t is_stmt = default_is_stmt;
-  bool basic_block = false;
-  bool prologue_end = false;
-  bool epilogue_begin = false;
-  unsigned int isa = 0;
-  unsigned int discriminator = 0;
+  state.is_stmt = default_is_stmt;
 
   /* Apply the "operation advance" from a special opcode or
      DW_LNS_advance_pc (as per DWARF4 6.2.5.1).  */
-  inline void advance_pc (unsigned int op_advance)
-  {
-    addr += minimum_instr_len * ((op_index + op_advance)
-				 / max_ops_per_instr);
-    op_index = (op_index + op_advance) % max_ops_per_instr;
-  }
+#define advance_pc(op_advance) \
+  run_advance_pc (&state, op_advance, minimum_instr_len, max_ops_per_instr)
 
   /* Process the instructions.  */
 
-  /* Adds a new line to the matrix.
-     We cannot simply define a function because we want to use alloca.  */
+  /* Adds a new line to the matrix.  For the first MAX_STACK_LINES
+     entries just return a slot in the preallocated stack array.  */
+  struct linelist llstack[MAX_STACK_LINES];
 #define NEW_LINE(end_seq)						\
   do {								\
-    struct linelist *ll = (nlinelist < MAX_STACK_ALLOC		\
-			   ? alloca (sizeof (struct linelist))	\
+    struct linelist *ll = (state.nlinelist < MAX_STACK_LINES	\
+			   ? &llstack[state.nlinelist]		\
 			   : malloc (sizeof (struct linelist)));	\
-    if (nlinelist >= MAX_STACK_ALLOC)				\
-      malloc_linelist = ll;						\
-    if (unlikely (add_new_line (ll, end_seq)))			\
+    if (unlikely (ll == NULL))					\
+      goto no_mem;						\
+    state.end_sequence = end_seq;				\
+    if (unlikely (add_new_line (&state, ll)))			\
       goto invalid_data;						\
   } while (0)
 
-  inline bool add_new_line (struct linelist *new_line, bool end_sequence)
-  {
-    new_line->next = linelist;
-    new_line->sequence = nlinelist;
-    linelist = new_line;
-    ++nlinelist;
-
-    /* Set the line information.  For some fields we use bitfields,
-       so we would lose information if the encoded values are too large.
-       Check just for paranoia, and call the data "invalid" if it
-       violates our assumptions on reasonable limits for the values.  */
-#define SET(field)							      \
-    do {								      \
-      new_line->line.field = field;					      \
-      if (unlikely (new_line->line.field != field))			      \
-	return true;						      \
-    } while (0)
-
-    SET (addr);
-    SET (op_index);
-    SET (file);
-    SET (line);
-    SET (column);
-    SET (is_stmt);
-    SET (basic_block);
-    SET (end_sequence);
-    SET (prologue_end);
-    SET (epilogue_begin);
-    SET (isa);
-    SET (discriminator);
-
-#undef SET
-
-    return false;
-  }
-
   while (linep < lineendp)
     {
       unsigned int opcode;
@@ -396,17 +455,17 @@ read_srclines (Dwarf *dbg,
 				+ (opcode - opcode_base) % line_range);
 
 	  /* Perform the increments.  */
-	  line += line_increment;
+	  state.line += line_increment;
 	  advance_pc ((opcode - opcode_base) / line_range);
 
 	  /* Add a new line with the current state machine values.  */
 	  NEW_LINE (0);
 
 	  /* Reset the flags.  */
-	  basic_block = false;
-	  prologue_end = false;
-	  epilogue_begin = false;
-	  discriminator = 0;
+	  state.basic_block = false;
+	  state.prologue_end = false;
+	  state.epilogue_begin = false;
+	  state.discriminator = 0;
 	}
       else if (opcode == 0)
 	{
@@ -431,28 +490,28 @@ read_srclines (Dwarf *dbg,
 	      NEW_LINE (1);
 
 	      /* Reset the registers.  */
-	      addr = 0;
-	      op_index = 0;
-	      file = 1;
-	      line = 1;
-	      column = 0;
-	      is_stmt = default_is_stmt;
-	      basic_block = false;
-	      prologue_end = false;
-	      epilogue_begin = false;
-	      isa = 0;
-	      discriminator = 0;
+	      state.addr = 0;
+	      state.op_index = 0;
+	      state.file = 1;
+	      state.line = 1;
+	      state.column = 0;
+	      state.is_stmt = default_is_stmt;
+	      state.basic_block = false;
+	      state.prologue_end = false;
+	      state.epilogue_begin = false;
+	      state.isa = 0;
+	      state.discriminator = 0;
 	      break;
 
 	    case DW_LNE_set_address:
 	      /* The value is an address.  The size is defined as
 		 apporiate for the target machine.  We use the
 		 address size field from the CU header.  */
-	      op_index = 0;
+	      state.op_index = 0;
 	      if (unlikely (lineendp - linep < (uint8_t) address_size))
 		goto invalid_data;
 	      if (__libdw_read_address_inc (dbg, IDX_debug_line, &linep,
-					    address_size, &addr))
+					    address_size, &state.addr))
 		goto out;
 	      break;
 
@@ -483,33 +542,29 @@ read_srclines (Dwarf *dbg,
 		  goto invalid_data;
 		get_uleb128 (filelength, linep, lineendp);
 
-		struct filelist *new_file =
-		  (struct filelist *) alloca (sizeof (*new_file));
+		struct filelist *new_file = NEW_FILE ();
 		if (fname[0] == '/')
 		  new_file->info.name = fname;
 		else
 		  {
 		    new_file->info.name =
-		      libdw_alloc (dbg, char, 1, (dirarray[diridx]->len + 1
+		      libdw_alloc (dbg, char, 1, (dirarray[diridx].len + 1
 						  + fnamelen + 1));
 		    char *cp = new_file->info.name;
 
-		    if (dirarray[diridx]->dir != NULL)
+		    if (dirarray[diridx].dir != NULL)
 		      /* This value could be NULL in case the
 			 DW_AT_comp_dir was not present.  We
 			 cannot do much in this case.  The easiest
 			 thing is to convert the path in an
 			 absolute path.  */
-		      cp = stpcpy (cp, dirarray[diridx]->dir);
+		      cp = stpcpy (cp, dirarray[diridx].dir);
 		    *cp++ = '/';
 		    strcpy (cp, fname);
 		  }
 
 		new_file->info.mtime = mtime;
 		new_file->info.length = filelength;
-		new_file->next = filelist;
-		filelist = new_file;
-		++nfilelist;
 	      }
 	      break;
 
@@ -520,7 +575,7 @@ read_srclines (Dwarf *dbg,
 
 	      if (unlikely (linep >= lineendp))
 		goto invalid_data;
-	      get_uleb128 (discriminator, linep, lineendp);
+	      get_uleb128 (state.discriminator, linep, lineendp);
 	      break;
 
 	    default:
@@ -545,10 +600,10 @@ read_srclines (Dwarf *dbg,
 	      NEW_LINE (0);
 
 	      /* Reset the flags.  */
-	      basic_block = false;
-	      prologue_end = false;
-	      epilogue_begin = false;
-	      discriminator = 0;
+	      state.basic_block = false;
+	      state.prologue_end = false;
+	      state.epilogue_begin = false;
+	      state.discriminator = 0;
 	      break;
 
 	    case DW_LNS_advance_pc:
@@ -572,7 +627,7 @@ read_srclines (Dwarf *dbg,
 	      if (unlikely (linep >= lineendp))
 		goto invalid_data;
 	      get_sleb128 (s128, linep, lineendp);
-	      line += s128;
+	      state.line += s128;
 	      break;
 
 	    case DW_LNS_set_file:
@@ -583,7 +638,7 @@ read_srclines (Dwarf *dbg,
 	      if (unlikely (linep >= lineendp))
 		goto invalid_data;
 	      get_uleb128 (u128, linep, lineendp);
-	      file = u128;
+	      state.file = u128;
 	      break;
 
 	    case DW_LNS_set_column:
@@ -594,7 +649,7 @@ read_srclines (Dwarf *dbg,
 	      if (unlikely (linep >= lineendp))
 		goto invalid_data;
 	      get_uleb128 (u128, linep, lineendp);
-	      column = u128;
+	      state.column = u128;
 	      break;
 
 	    case DW_LNS_negate_stmt:
@@ -602,7 +657,7 @@ read_srclines (Dwarf *dbg,
 	      if (unlikely (standard_opcode_lengths[opcode] != 0))
 		goto invalid_data;
 
-	      is_stmt = 1 - is_stmt;
+	      state.is_stmt = 1 - state.is_stmt;
 	      break;
 
 	    case DW_LNS_set_basic_block:
@@ -610,7 +665,7 @@ read_srclines (Dwarf *dbg,
 	      if (unlikely (standard_opcode_lengths[opcode] != 0))
 		goto invalid_data;
 
-	      basic_block = true;
+	      state.basic_block = true;
 	      break;
 
 	    case DW_LNS_const_add_pc:
@@ -631,8 +686,8 @@ read_srclines (Dwarf *dbg,
 		  || unlikely (lineendp - linep < 2))
 		goto invalid_data;
 
-	      addr += read_2ubyte_unaligned_inc (dbg, linep);
-	      op_index = 0;
+	      state.addr += read_2ubyte_unaligned_inc (dbg, linep);
+	      state.op_index = 0;
 	      break;
 
 	    case DW_LNS_set_prologue_end:
@@ -640,7 +695,7 @@ read_srclines (Dwarf *dbg,
 	      if (unlikely (standard_opcode_lengths[opcode] != 0))
 		goto invalid_data;
 
-	      prologue_end = true;
+	      state.prologue_end = true;
 	      break;
 
 	    case DW_LNS_set_epilogue_begin:
@@ -648,7 +703,7 @@ read_srclines (Dwarf *dbg,
 	      if (unlikely (standard_opcode_lengths[opcode] != 0))
 		goto invalid_data;
 
-	      epilogue_begin = true;
+	      state.epilogue_begin = true;
 	      break;
 
 	    case DW_LNS_set_isa:
@@ -658,7 +713,7 @@ read_srclines (Dwarf *dbg,
 
 	      if (unlikely (linep >= lineendp))
 		goto invalid_data;
-	      get_uleb128 (isa, linep, lineendp);
+	      get_uleb128 (state.isa, linep, lineendp);
 	      break;
 	    }
 	}
@@ -687,25 +742,27 @@ read_srclines (Dwarf *dbg,
 				    1);
   const char **dirs = (void *) &files->info[nfilelist];
 
+  struct filelist *fileslist = filelist;
   files->nfiles = nfilelist;
-  while (nfilelist-- > 0)
+  for (size_t n = nfilelist; n > 0; n--)
     {
-      files->info[nfilelist] = filelist->info;
-      filelist = filelist->next;
+      files->info[n - 1] = fileslist->info;
+      fileslist = fileslist->next;
     }
-  assert (filelist == NULL);
+  assert (fileslist == NULL);
 
   /* Put all the directory strings in an array.  */
   files->ndirs = ndirlist;
   for (unsigned int i = 0; i < ndirlist; ++i)
-    dirs[i] = dirarray[i]->dir;
+    dirs[i] = dirarray[i].dir;
   dirs[ndirlist] = NULL;
 
   /* Pass the file data structure to the caller.  */
   if (filesp != NULL)
     *filesp = files;
 
-  size_t buf_size = (sizeof (Dwarf_Lines) + (sizeof (Dwarf_Line) * nlinelist));
+  size_t buf_size = (sizeof (Dwarf_Lines)
+		     + (sizeof (Dwarf_Line) * state.nlinelist));
   void *buf = libdw_alloc (dbg, Dwarf_Lines, buf_size, 1);
 
   /* First use the buffer for the pointers, and sort the entries.
@@ -713,27 +770,28 @@ read_srclines (Dwarf *dbg,
      copy into the buffer from the beginning so the overlap works.  */
   assert (sizeof (Dwarf_Line) >= sizeof (struct linelist *));
   struct linelist **sortlines = (buf + buf_size
-				 - sizeof (struct linelist **) * nlinelist);
+				 - sizeof (struct linelist **) * state.nlinelist);
 
   /* The list is in LIFO order and usually they come in clumps with
      ascending addresses.  So fill from the back to probably start with
      runs already in order before we sort.  */
-  for (size_t i = nlinelist; i-- > 0; )
+  struct linelist *lineslist = state.linelist;
+  for (size_t i = state.nlinelist; i-- > 0; )
     {
-      sortlines[i] = linelist;
-      linelist = linelist->next;
+      sortlines[i] = lineslist;
+      lineslist = lineslist->next;
     }
-  assert (linelist == NULL);
+  assert (lineslist == NULL);
 
   /* Sort by ascending address.  */
-  qsort (sortlines, nlinelist, sizeof sortlines[0], &compare_lines);
+  qsort (sortlines, state.nlinelist, sizeof sortlines[0], &compare_lines);
 
   /* Now that they are sorted, put them in the final array.
      The buffers overlap, so we've clobbered the early elements
      of SORTLINES by the time we're reading the later ones.  */
   Dwarf_Lines *lines = buf;
-  lines->nlines = nlinelist;
-  for (size_t i = 0; i < nlinelist; ++i)
+  lines->nlines = state.nlinelist;
+  for (size_t i = 0; i < state.nlinelist; ++i)
     {
       lines->info[i] = sortlines[i]->line;
       lines->info[i].files = files;
@@ -742,8 +800,8 @@ read_srclines (Dwarf *dbg,
   /* Make sure the highest address for the CU is marked as end_sequence.
      This is required by the DWARF spec, but some compilers forget and
      dwfl_module_getsrc depends on it.  */
-  if (nlinelist > 0)
-    lines->info[nlinelist - 1].end_sequence = 1;
+  if (state.nlinelist > 0)
+    lines->info[state.nlinelist - 1].end_sequence = 1;
 
   /* Pass the line structure back to the caller.  */
   if (linesp != NULL)
@@ -754,11 +812,19 @@ read_srclines (Dwarf *dbg,
 
  out:
   /* Free malloced line records, if any.  */
-  for (size_t i = MAX_STACK_ALLOC; i < nlinelist; i++)
+  for (size_t i = MAX_STACK_LINES; i < state.nlinelist; i++)
     {
-      struct linelist *ll = malloc_linelist->next;
-      free (malloc_linelist);
-      malloc_linelist = ll;
+      struct linelist *ll = state.linelist->next;
+      free (state.linelist);
+      state.linelist = ll;
+    }
+  if (ndirlist >= MAX_STACK_DIRS)
+    free (dirarray);
+  for (size_t i = MAX_STACK_FILES; i < nfilelist; i++)
+    {
+      struct filelist *fl = filelist->next;
+      free (filelist);
+      filelist = fl;
     }
 
   return res;
@@ -838,10 +904,13 @@ __libdw_getcompdir (Dwarf_Die *cudie)
 int
 dwarf_getsrclines (Dwarf_Die *cudie, Dwarf_Lines **lines, size_t *nlines)
 {
-  if (unlikely (cudie == NULL
-		|| (INTUSE(dwarf_tag) (cudie) != DW_TAG_compile_unit
-		    && INTUSE(dwarf_tag) (cudie) != DW_TAG_partial_unit)))
+  if (cudie == NULL)
     return -1;
+  if (! is_cudie (cudie))
+    {
+      __libdw_seterrno (DWARF_E_NOT_CUDIE);
+      return -1;
+    }
 
   /* Get the information if it is not already known.  */
   struct Dwarf_CU *const cu = cudie->cu;
