@@ -101,6 +101,29 @@ sort_sections (Elf_Scn **scns, Elf_ScnList *list)
 }
 
 
+static inline void
+fill_mmap (size_t offset, char *last_position, char *scn_start,
+           char *const shdr_start, char *const shdr_end)
+{
+  size_t written = 0;
+
+  if (last_position < shdr_start)
+    {
+      written = MIN (scn_start + offset - last_position,
+                     shdr_start - last_position);
+
+      memset (last_position, __libelf_fill_byte, written);
+    }
+
+  if (last_position + written != scn_start + offset
+      && shdr_end < scn_start + offset)
+    {
+      char *fill_start = MAX (shdr_end, scn_start);
+      memset (fill_start, __libelf_fill_byte,
+              scn_start + offset - fill_start);
+    }
+}
+
 int
 internal_function
 __elfw2(LIBELFBITS,updatemmap) (Elf *elf, int change_bo, size_t shnum)
@@ -303,27 +326,6 @@ __elfw2(LIBELFBITS,updatemmap) (Elf *elf, int change_bo, size_t shnum)
 	  Elf_Data_List *dl = &scn->data_list;
 	  bool scn_changed = false;
 
-	  void fill_mmap (size_t offset)
-	  {
-	    size_t written = 0;
-
-	    if (last_position < shdr_start)
-	      {
-		written = MIN (scn_start + offset - last_position,
-			       shdr_start - last_position);
-
-		memset (last_position, __libelf_fill_byte, written);
-	      }
-
-	    if (last_position + written != scn_start + offset
-		&& shdr_end < scn_start + offset)
-	      {
-		char *fill_start = MAX (shdr_end, scn_start);
-		memset (fill_start, __libelf_fill_byte,
-			scn_start + offset - fill_start);
-	      }
-	  }
-
 	  if (scn->data_list_rear != NULL)
 	    do
 	      {
@@ -338,7 +340,8 @@ __elfw2(LIBELFBITS,updatemmap) (Elf *elf, int change_bo, size_t shnum)
 			|| ((scn->flags | dl->flags | elf->flags)
 			    & ELF_F_DIRTY) != 0))
 		  {
-		    fill_mmap (dl->data.d.d_off);
+		    fill_mmap (dl->data.d.d_off, last_position, scn_start,
+		               shdr_start, shdr_end);
 		    last_position = scn_start + dl->data.d.d_off;
 		  }
 
@@ -390,7 +393,8 @@ __elfw2(LIBELFBITS,updatemmap) (Elf *elf, int change_bo, size_t shnum)
 	      /* If the previous section (or the ELF/program
 		 header) changed we might have to fill the gap.  */
 	      if (scn_start > last_position && previous_scn_changed)
-		fill_mmap (0);
+		fill_mmap (0, last_position, scn_start,
+		           shdr_start, shdr_end);
 
 	      /* We have to trust the existing section header information.  */
 	      last_position = scn_start + shdr->sh_size;
@@ -655,16 +659,18 @@ __elfw2(LIBELFBITS,updatefile) (Elf *elf, int change_bo, size_t shnum)
 #endif
 
       ElfW2(LIBELFBITS,Shdr) *shdr_data;
+      ElfW2(LIBELFBITS,Shdr) *shdr_data_mem = NULL;
       if (change_bo || elf->state.ELFW(elf,LIBELFBITS).shdr == NULL
 	  || (elf->flags & ELF_F_DIRTY))
 	{
-	  shdr_data = (ElfW2(LIBELFBITS,Shdr) *)
+	  shdr_data_mem = (ElfW2(LIBELFBITS,Shdr) *)
 	    malloc (shnum * sizeof (ElfW2(LIBELFBITS,Shdr)));
-	  if (unlikely (shdr_data == NULL))
+	  if (unlikely (shdr_data_mem == NULL))
 	    {
 	      __libelf_seterrno (ELF_E_NOMEM);
 	      return -1;
 	    }
+	  shdr_data = shdr_data_mem;
 	}
       else
 	shdr_data = elf->state.ELFW(elf,LIBELFBITS).shdr;
@@ -675,6 +681,7 @@ __elfw2(LIBELFBITS,updatefile) (Elf *elf, int change_bo, size_t shnum)
       Elf_Scn **scns = (Elf_Scn **) malloc (shnum * sizeof (Elf_Scn *));
       if (unlikely (scns == NULL))
 	{
+	  free (shdr_data_mem);
 	  __libelf_seterrno (ELF_E_NOMEM);
 	  return -1;
 	}
@@ -712,7 +719,12 @@ __elfw2(LIBELFBITS,updatefile) (Elf *elf, int change_bo, size_t shnum)
 					(scn_start + dl->data.d.d_off)
 					- last_offset, fillbuf,
 					&filled) != 0))
-		      return 1;
+		      {
+		      fail_free:
+			free (shdr_data_mem);
+			free (scns);
+			return 1;
+		      }
 		  }
 
 		if ((scn->flags | dl->flags | elf->flags) & ELF_F_DIRTY)
@@ -744,7 +756,7 @@ __elfw2(LIBELFBITS,updatefile) (Elf *elf, int change_bo, size_t shnum)
 			    if (unlikely (buf == NULL))
 			      {
 				__libelf_seterrno (ELF_E_NOMEM);
-				return 1;
+				goto fail_free;
 			      }
 			  }
 
@@ -761,7 +773,7 @@ __elfw2(LIBELFBITS,updatefile) (Elf *elf, int change_bo, size_t shnum)
 			  free (buf);
 
 			__libelf_seterrno (ELF_E_WRITE_ERROR);
-			return 1;
+			goto fail_free;
 		      }
 
 		    if (buf != dl->data.d.d_buf && buf != tmpbuf)
@@ -786,7 +798,7 @@ __elfw2(LIBELFBITS,updatefile) (Elf *elf, int change_bo, size_t shnum)
 		  if (unlikely (fill (elf->fildes, last_offset,
 				      scn_start - last_offset, fillbuf,
 				      &filled) != 0))
-		    return 1;
+		    goto fail_free;
 		}
 
 	      last_offset = scn_start + shdr->sh_size;
@@ -814,7 +826,7 @@ __elfw2(LIBELFBITS,updatefile) (Elf *elf, int change_bo, size_t shnum)
 	  && unlikely (fill (elf->fildes, last_offset,
 			     shdr_offset - last_offset,
 			     fillbuf, &filled) != 0))
-	return 1;
+	goto fail_free;
 
       /* Write out the section header table.  */
       if (shdr_flags & ELF_F_DIRTY
@@ -824,13 +836,10 @@ __elfw2(LIBELFBITS,updatefile) (Elf *elf, int change_bo, size_t shnum)
 		       != sizeof (ElfW2(LIBELFBITS,Shdr)) * shnum))
 	{
 	  __libelf_seterrno (ELF_E_WRITE_ERROR);
-	  return 1;
+	  goto fail_free;
 	}
 
-      if (change_bo || elf->state.ELFW(elf,LIBELFBITS).shdr == NULL
-	  || (elf->flags & ELF_F_DIRTY))
-	free (shdr_data);
-
+      free (shdr_data_mem);
       free (scns);
     }
 
