@@ -28,7 +28,6 @@
 #include <libintl.h>
 #include <limits.h>
 #include <locale.h>
-#include <mcheck.h>
 #include <search.h>
 #include <stdbool.h>
 #include <stdlib.h>
@@ -141,9 +140,6 @@ static enum { ipos_none, ipos_before, ipos_after } ipos;
 int
 main (int argc, char *argv[])
 {
-  /* Make memory leak detection possible.  */
-  mtrace ();
-
   /* We use no threads here which can interfere with handling a stream.  */
   (void) __fsetlocking (stdin, FSETLOCKING_BYCALLER);
   (void) __fsetlocking (stdout, FSETLOCKING_BYCALLER);
@@ -683,13 +679,13 @@ do_oper_extract (int oper, const char *arfname, char **argv, int argc,
 
 	      if (preserve_dates)
 		{
-		  struct timeval tv[2];
+		  struct timespec tv[2];
 		  tv[0].tv_sec = arhdr->ar_date;
-		  tv[0].tv_usec = 0;
+		  tv[0].tv_nsec = 0;
 		  tv[1].tv_sec = arhdr->ar_date;
-		  tv[1].tv_usec = 0;
+		  tv[1].tv_nsec = 0;
 
-		  if (unlikely (futimes (xfd, tv) != 0))
+		  if (unlikely (futimens (xfd, tv) != 0))
 		    {
 		      error (0, errno,
 			     gettext ("cannot change modification time of %s"),
